#DPL1

#general function for LDA

#----------------------------------------------------------------

LDA <- function(X,X_tune,n,n_tune,p,labelX,labelX_tune,
        BIC,num_cluster,lambda,rho,tol,ite_max){
   
#----------------------------------------------------------------

# #standardize
# mu_X <- apply(X,2,mean)
# X <- t(t(X)-mu_X)
# sd_X <- apply(X,2,sd)
# X <- t(t(X)/sd_X)
# mu_Xtune <- apply(X_tune,2,mean)
# X_tune <- t(t(X_tune)-mu_Xtune)
# sd_Xtune <- apply(X,2,sd)
# X_tune <- t(t(X_tune)/sd_Xtune)

#----------------------------------------------------------------

nGlob <- n 
pGlob <- p
KGlob <- num_cluster

#----------------------------------------------------------------

lambda <- sort(lambda, decreasing=T)
rho <- sort(rho, decreasing=T)
# possible values of penalization parameters
lam1_num <- length(lambda)
rho_num <- length(rho)
 
#----------------------------------------------------------------

#record (-1)*predicitve log-likelihood or BIC value
pred_likelihood <- array(-1e5, dim=c(lam1_num,rho_num))
#record mean of X based on classification
classify.mu <- array(0, dim=c(lam1_num,rho_num,KGlob*pGlob))
#record precision matrix of X based on classification
classify.Omega <- array(0, dim=c(lam1_num,rho_num,pGlob^2))

#----------------------------------------------------------------

#matrix TauX representing label based on X
TauX <- matrix(0, nGlob, KGlob)
for(i in 1:n) {
TauX[i,labelX[i]] <- 1
}
#vector WX representing proportion
WX <- rep(0,KGlob)
for(k in 1:KGlob) {
WX[k] <- sum(labelX==k) / nGlob
}
#vector WX_tune representing proportion
WX_tune <- rep(0,KGlob)
for(k in 1:KGlob) {
WX_tune[k] <- sum(labelX_tune==k) / (dim(X_tune)[1])
}

#----------------------------------------------------------------

#initial values
ini.out <- ini(X,KGlob,labelX)
mu.ini <- ini.out$mu
sigma.ini <- ini.out$sigma

for(lam1 in 1:lam1_num) {
 lambda_c <- lambda[lam1]
 mu_F <- mu.ini
 Omega_F <- diag(1/sigma.ini)
 for(lam2 in 1:rho_num) {
  rho_c <- rho[lam2]
  #-------------------------------------------------------------    
  #iterative algorithm
  out <- Iterate(X,X_tune,nGlob,n_tune,pGlob,KGlob,labelX,
                 labelX_tune,WX,WX_tune,TauX,lambda_c,rho_c,mu_F,
		         Omega_F,tol,ite_max,BIC) 
  mu_F <- out$mu_F
  Omega_F <- out$Omega_F
  #-------------------------------------------------------------   
  classify.mu[lam1,lam2,] <- vec(mu_F)
  classify.Omega[lam1,lam2,] <- vec(Omega_F)
  #calculate the likelihood based on the tuning data
  if(BIC==T) {
   plog <- ploglik(X,mu_F,WX,Omega_F,labelX,KGlob,nGlob,pGlob,BIC)
  }
  if(BIC==F) {
   plog <- ploglik(X_tune,mu_F,WX_tune,Omega_F,labelX_tune,
                   KGlob,n_tune,pGlob,BIC)
  }
  pred_likelihood[lam1,lam2] <- plog
  print(c(plog, lam1,lam2))
 }
}  

#----------------------------------------------------------------

target <- pred_likelihood[1,1]
target_m <- 1
target_c <- 1
for(lam1 in 1:lam1_num) {
 for(lam2 in 1:rho_num) {
  if(target > pred_likelihood[lam1,lam2]) {
   target <- pred_likelihood[lam1,lam2]
   target_m <- lam1
   target_c <- lam2
  }
 }
}

#----------------------------------------------------------------

lambda_F <- lambda[target_m]
rho_F <- rho[target_c]
vec_mean <- classify.mu[target_m,target_c,]
group_mean <- invvec(vec_mean,pGlob)
vec_Omega <- classify.Omega[target_m,target_c,]
group_Cov <- invvec(vec_Omega,pGlob)

#----------------------------------------------------------------

group_member <- rep(NA,nGlob)
for(i in 1:nGlob) {
postpro <- rep(NA, KGlob)
for(k in 1:KGlob) {
 postpro[k] <- WX[k]*mvnf(X[i,], group_mean[k,], group_Cov)
}
group_member[i] <- which.max(postpro/sum(postpro))
}

#----------------------------------------------------------------

output <- list(lambda=lambda_F,rho=rho_F,group_member=group_member,
                mean=group_mean,Cov=group_Cov)
			   # pred_likelihood=pred_likelihood,
			   # classify.mu=classify.mu,
			   # classify.Omega=classify.Omega)
		   
return(output)

}

