#Sparse discriminate analysis
#Clemmensen et al. (2011)

#Use BIC to select tuning parameters (lambda2, lamda1)
#lambda2 --- corresponding to L2 norm
#lambda1 --- corresponding to L1 norm


library(sparseLDA)

#----------------------------------------------------------------

SDA <- function(X, X_tune, Y, Y_tune, Xtest, lambda2, lambda1) {

#----------------------------------------------------------------

n <- dim(X)[1]
n_tune <- dim(X_tune)[1]
colnames(Y) <- c("A", "B", "C")

#----------------------------------------------------------------

#Normalize data
Xc <- normalize(X)
Xn <- Xc$Xc
p <- dim(Xn)[2]
Xn_tune <- normalizetest(X_tune, Xc)

#----------------------------------------------------------------

nlambda2 <- length(lambda2)
nlambda1 <- length(lambda1)
score <- matrix(NA, nlambda2, nlambda1)  #record BIC values
result <- list()
#Perform SDA given lambda2 and lambda1
for(i in 1:nlambda2) {
 for(j in 1:nlambda1) {
  out <- sda(Xn, Y, lambda=lambda2[i],
                                 stop=-lambda1[j], maxIte=25)
  result[[(i-1)*nlambda1+j]] <- out
  RSS <- sum((Y_tune%*%out$theta-Xn_tune[,out$varIndex]%*%out$beta)^2)
  score[i,j] <- n*log(sum(RSS)/n) + log(n)*length(out$varIndex)
  #print(c(lambda2[i],lambda1[j],score[i,j]))
 }
}
ind <- as.vector(which(score==min(score), arr.ind=TRUE))
i_opt <- ind[1]
j_opt <- ind[2]
#print(c(lambda1[j_opt], lambda2[i_opt]))

#----------------------------------------------------------------

#SDA based on the optimal lambda2 and lambda1
out <- result[[(i_opt-1)*nlambda1+j_opt]]  
print(c(lambda1[j_opt], lambda2[i_opt], sum(out$beta[,1]!=0), sum(out$beta[,2]!=0)))
#predict training samples
train <- predict(out, Xn)
label_train <- as.numeric(train$class)
#testing
Xtst <- normalizetest(Xtest, Xc)
test <- predict(out, Xtst)
label_test <- as.numeric(test$class)

#----------------------------------------------------------------

return(list(varIndex=out$varIndex, label_train=label_train, 
            label_test=label_test))

}

    