#DPL1

# common unstrained covariance

# useful functions

#----------------------------------------------------------------

#vec function
vec <- function(A)  {
p <- dim(A)[2]
out <- A[,1]
for(j in 2:p) {
 out <- c(out, A[,j])
}
out
}

#----------------------------------------------------------------

#transform a vector to a matrix
invvec <- function(a,p)  {
length <- dim(t(a))[2]
num.row <- length/p
A <- matrix(0,num.row, p)
for(j in 1:p) {
A[ ,j] <- a[((j-1)*num.row+1):(j*num.row)]
}
A
}

#----------------------------------------------------------------

#the kernel of MVN(mean,Sigma)'s pdf, where Omega=inverse of Simga
mvnf <- function(x, mean, Omega) {
value <- exp(-0.5 * t(x-mean) %*% Omega %*% (x-mean))
return(value)
}

#----------------------------------------------------------------

ploglik <- function(X,mu,W,Omega,label,KGlob,nGlob,pGlob,BIC) {

plog <- 0
for(i in 1:nGlob) {
labeli <- label[i]
plog <- plog + log(W[labeli] * mvnf(X[i,], mu[labeli,], Omega))
}
eigenv <- svd(Omega)$d
eigenv[eigenv<1e-10] <- 1e-10
plog <- 0.5*nGlob*sum(log(eigenv)) + plog

plog <- -2*plog

if(BIC==T)
{
BIC_penal <- (KGlob-1)+(KGlob*pGlob-sum(mu==0))+(pGlob*(pGlob+1)/2-sum(Omega==0)/2) 
plog <- plog+log(nGlob)*BIC_penal
}

return(plog)
}

#----------------------------------------------------------------

#initial value of estimators
ini <- function(X,KGlob,label) {

#----------------------------------------------------------------

nGlob <- dim(X)[1]
pGlob <- dim(X)[2]

#----------------------------------------------------------------

mu <- matrix(NA, KGlob, pGlob)
for(k in 1:KGlob) {
indk <- c(1:nGlob)[label==k]
mu[k,] <- colSums(X[indk,])
mu[k,] <- mu[k,] / (sum(label==k))
}

#----------------------------------------------------------------

#the variance of each variable
sigma_ini <- rep(0, pGlob)
for(j in 1:pGlob) {
 for(i in 1:nGlob) {
  sigma_ini[j] <- sigma_ini[j]+(X[i,j]-mu[label[i],j])^2
 }
}
sigma_ini <- sigma_ini / nGlob

#----------------------------------------------------------------

return(list(mu=mu,sigma=sigma_ini))

}

#----------------------------------------------------------------

#update Simga or precision matrix Omega
Omega.update <- function(X,mu.old,Tau,rho) {

#----------------------------------------------------------------

KGlob <- dim(Tau)[2]
nGlob <- dim(X)[1]
pGlob <- dim(X)[2]

#----------------------------------------------------------------

#ind <- 1
#sample covariance matrix
S <- matrix(0, pGlob, pGlob)
for(i in 1:nGlob) {
 for(k in 1:KGlob) {
  S <- S + Tau[i,k] * (X[i,]-mu.old[k,]) %*% t(X[i,]-mu.old[k,])
 }
}
S <- S / nGlob
rho_c <- rho / nGlob

#----------------------------------------------------------------

#use glasso algorithm
out <- glasso(S,rho=rho_c,maxit=100,approx=F,penalize.diagonal=F,trace=FALSE)
Omega <- out$wi
#print(c(min(Omega), max(Omega), sum(Omega==0)))

#----------------------------------------------------------------

return(Omega)

}

#----------------------------------------------------------------

#the updated mean parameter matrix mu
#given Omega
Mu.update <- function(X,mu.old,Tau,Omega.new,lambda) {

#----------------------------------------------------------------

KGlob <- dim(Tau)[2]
nGlob <- dim(X)[1]
pGlob <- dim(X)[2]
lambda_c <- lambda 
mu.new <- matrix(NA, KGlob, pGlob)

#----------------------------------------------------------------

for(k in 1:KGlob){
 for(j in 1:pGlob){
  total <- 0
  for(i in 1:nGlob) {
   total <- total+Tau[i,k]*( (X[i,-j]-mu.old[k,-j])%*%Omega.new[j,-j] + Omega.new[j,j]*X[i,j] )
  }
  if(abs(total)<=lambda_c) { 
   mu.new[k,j] <- 0 
  } else {
   temp <- sum(Tau[,k]*(X%*%Omega.new[,j])) - sum(Tau[,k])*(t(mu.old[k,])%*%Omega.new[,j]-mu.old[k,j]*Omega.new[j,j])
   if(temp>=0){
    mu.new[k,j] <- (temp-lambda_c)/sum(Tau[,k])/Omega.new[j,j] 
   } else { 
    mu.new[k,j] <- (temp+lambda_c)/sum(Tau[,k])/Omega.new[j,j] 
   }
  }
 }
}

#----------------------------------------------------------------

return(mu.new)

}

