#LDA

# common unstrained covariance
# useful functions

#----------------------------------------------------------------

#vec function
vec <- function(A)  {
p <- dim(A)[2]
out <- A[,1]
for(j in 2:p) {
 out <- c(out, A[,j])
}
out
}

#----------------------------------------------------------------

#transform a vector to a matrix
invvec <- function(a,p)  {
length <- dim(t(a))[2]
num.row <- length/p
A <- matrix(0,num.row, p)
for(j in 1:p) {
A[ ,j] <- a[((j-1)*num.row+1):(j*num.row)]
}
A
}

#----------------------------------------------------------------


#the kernel of MVN(mean,Sigma)'s pdf, where Omega=inverse of Simga
mvnf <- function(x, mean, Omega) {
value <- exp(-0.5 * t(x-mean) %*% Omega %*% (x-mean))
return(value)
}

#----------------------------------------------------------------

ploglik <- function(X,mu,W,Omega,label,KGlob,nGlob,pGlob,BIC) {

plog <- 0
for(i in 1:nGlob) {
labeli <- label[i]
plog <- plog + log(W[labeli] * mvnf(X[i,], mu[labeli,], Omega))
}
eigenv <- svd(Omega)$d
eigenv[eigenv<1e-10] <- 1e-10
plog <- 0.5*nGlob*sum(log(eigenv)) + plog

plog <- -2*plog

if(BIC==T)
{
if(sum(mu==0) == 0) {
BIC_penal <- (KGlob-1)+(length(unique(as.vector(mu))))+(pGlob*(pGlob+1)/2-sum(Omega==0)/2) 
}
if(sum(mu==0) > 0) {
BIC_penal <- (KGlob-1)+(length(unique(as.vector(mu))) - 1)+(pGlob*(pGlob+1)/2-sum(Omega==0)/2) 
}
plog <- plog+log(nGlob)*BIC_penal
}

return(plog)
}


#----------------------------------------------------------------

#initial value of estimators
ini <- function(X,KGlob,label) {

#----------------------------------------------------------------

nGlob <- dim(X)[1]
pGlob <- dim(X)[2]

#----------------------------------------------------------------

mu <- matrix(NA, KGlob, pGlob)
for(k in 1:KGlob) {
indk <- c(1:nGlob)[label==k]
mu[k,] <- colSums(X[indk,])
mu[k,] <- mu[k,] / (sum(label==k))
}

#----------------------------------------------------------------

#the variance of each variable
sigma_ini <- rep(0, pGlob)
for(j in 1:pGlob) {
 for(i in 1:nGlob) {
  sigma_ini[j] <- sigma_ini[j]+(X[i,j]-mu[label[i],j])^2
 }
}
sigma_ini <- sigma_ini / nGlob

#----------------------------------------------------------------

return(list(mu=mu,sigma=sigma_ini))

}


#----------------------------------------------------------------


#update Simga or precision matrix Omega
Omega.update <- function(X,mu.old,Tau,rho) {

#----------------------------------------------------------------

KGlob <- dim(Tau)[2]
nGlob <- dim(X)[1]
pGlob <- dim(X)[2]

#----------------------------------------------------------------

#ind <- 1
#sample covariance matrix
S <- matrix(0, pGlob, pGlob)
for(i in 1:nGlob) {
 for(k in 1:KGlob) {
  S <- S + Tau[i,k] * (X[i,]-mu.old[k,]) %*% t(X[i,]-mu.old[k,])
 }
}
S <- S / nGlob
rho_c <- rho / nGlob

#----------------------------------------------------------------

#use glasso algorithm
out <- glasso(S,rho=rho_c,maxit=100,approx=F,penalize.diagonal=F,trace=FALSE)
Omega <- out$wi
#print(c(min(Omega), max(Omega), sum(Omega==0)))

#----------------------------------------------------------------

return(Omega)

}


#----------------------------------------------------------------


#the updated mean parameter matrix mu
#given the updated Omega
#use r package: SparseM to solve linear equations
MuSM.update <- function(X,mu.old,Tau,W,Omega.new,
              mu.ave,lambda,tol_mu,ite_mu) {

#----------------------------------------------------------------

KGlob <- dim(Tau)[2]
nGlob <- dim(X)[1]
pGlob <- dim(X)[2]
lambda_c <- lambda / nGlob

#----------------------------------------------------------------

#weight matrix in pairwise penalty
Delta <- matrix(0, KGlob^2, pGlob)
for(k in 1:(KGlob-1)) {
 for(l in (k+1):KGlob){
   for(j in 1:pGlob) {
    # threshold the elements of denominator by 10^{-10}
    diffkl <- abs(mu.ave[k,j] - mu.ave[l,j])
    if(diffkl < 1e-10) {diffkl <- 1e-10}
    Delta[((l-1)*KGlob+k),j] <- Delta[((k-1)*KGlob+l),j] <- 1 / diffkl
  }
 }
}

#----------------------------------------------------------------

ind <- 1
#right hand vector
b <- vec(t(Tau) %*% X %*% Omega.new) / nGlob
A <- diag(W) 

#----------------------------------------------------------------

while(ind <= ite_mu) {

#----------------------------------------------------------------

#construct the linear equations for mu:
#(Omega kronecker A + B.old) mu = b

#left hand matrix:LHM
delta.old <- matrix(0, KGlob^2, pGlob)
for(k in 1:(KGlob-1)) {
 for(l in (k+1):KGlob) {
  for(j in 1:pGlob) {
   # threshold the elements of denominator by 10^{-6}
   diffkl <- abs(mu.old[k,j] - mu.old[l,j])
   if(diffkl < 1e-10) {diffkl <- 1e-10}
   delta.old[((l-1)*KGlob+k),j] <- delta.old[((k-1)*KGlob+l),j] <- Delta[((k-1)*KGlob+l),j] / (2*diffkl)
  }
 }
}

B.old <- matrix(0, KGlob*pGlob, KGlob*pGlob)
for(j in 1:pGlob) {
Bj <- matrix(0, KGlob, KGlob) 
for(u in 2:KGlob) {
 for(v in 1:(u-1)) {
  Bj[v,u] <- Bj[u,v] <- (-1) * delta.old[((v-1)*KGlob+u),j]
 }
}
for(s in 1:KGlob) {
Bj[s,s] <- sum(delta.old[((s-1)*KGlob+1):(s*KGlob), j]) - delta.old[(s-1)*KGlob+s,j]
}
Bj <- 2*lambda_c*Bj
B.old[((j-1)*KGlob+1):(j*KGlob), ((j-1)*KGlob+1):(j*KGlob)] <- Bj
}

LHM <- kronecker(Omega.new, A) + B.old

#----------------------------------------------------------------

# use SparseM to update mu
lh <- as.matrix.csr(LHM)
rh <- as.matrix(b, rnow=KGlob*pGlob)
vec.mu <- solve(lh, rh)

mu.new <- matrix(NA, KGlob, pGlob)
for(j in 1:pGlob) {
mu.new[ ,j] <- vec.mu[((j-1)*KGlob+1):(j*KGlob)]
}

#----------------------------------------------------------------

norm <- sum(abs(mu.new - mu.old)) / sum(abs(mu.old))
#print(c(norm, ind))

#--------------------------------------------------------------------------------
        
if (norm < tol_mu){ break }

ind <- ind + 1
mu.old <- mu.new

}

#print(c(norm, ind))

#--------------------------------------------------------------------------------

for(j in 1:pGlob) {
 for(k in 1:KGlob) { 
  diff.muj <- abs(mu.new[k,j] - mu.new[,j])
  #print(c(j, diff.muj))
  equ.count <- sum(diff.muj < 1e-3)
  if(equ.count > 1) { 
   index <- which(diff.muj < 1e-3)
   mu.new[index,j] <- sum(mu.new[index,j]) / equ.count
  }
 }
}

#--------------------------------------------------------------------------------

return(mu.new)

}


#--------------------------------------------------------------------------------


